<?php
/*
Plugin Name:	Blogfa Templater
Description:	Enables you to use blogfa.com templates in WordPress.
Author:			Hassan Derakhshandeh
Version:		0.1.1
Author URI:		http://20script.ir/


		* 	Copyright (C) 2011  Hassan Derakhshandeh
		*	http://20script.ir/
		*	hassan.derakhshandeh@gmail.com

		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class Blogfa_Template_Renderer {

	private $textdomain;

	function Blogfa_Template_Renderer() {
		add_action( 'admin_init', array( &$this, 'register_settings' ) );
		add_action( 'admin_menu', array( &$this, 'admin' ) );
		add_action( 'template_redirect', array( &$this, 'render' ), 10000 ); /* very low priority to allow other plugins to queue their style/script files */
		add_action( 'init', array( &$this, 'locale' ) );

		include_once( dirname( __FILE__ ) . '/general-template.php' );
	}

	function activation() {
		$options = get_option( 'blogfa', array() );
		$options['previous_template'] = get_option( 'template' );
		$options['previous_stylesheet'] = get_option( 'stylesheet' );
		/* this will effectivly disable all themes */
		update_option( 'template', '' );
		update_option( 'stylesheet', '' );
		update_option( 'blogfa', $options );
	}

	function deactivation() {
		$options = get_option( 'blogfa' );
		/* restore current theme */
		if( ! empty( $options['previous_template'] ) )
			update_option( 'template', $options['previous_template'] );
		if( ! empty( $options['previous_stylesheet'] ) )
			update_option( 'stylesheet', $options['previous_stylesheet'] );
	}

	function locale() {
		$this->textdomain = 'btr';
		load_plugin_textdomain( $this->textdomain, false, basename( dirname( __FILE__ ) ) . '/langs' );
	}

	function admin() {
		$options_page = add_theme_page(
			__( 'Blogfa Options', $this->textdomain ), // Name of page
			__( 'Blogfa Options', $this->textdomain ), // Label in menu
			'edit_theme_options',                  // Capability required
			'blogfa-options',                       // Menu slug, used to uniquely identify the page
			array( &$this, 'options_page' )            // Function that renders the options page
		);
		add_action( "admin_print_styles-{$options_page}", array( &$this, 'admin_queue' ) );
	}

	function options_page() {
		$options = get_option( 'blogfa' );
		include_once( dirname( __FILE__ ) . '/screens/options.php' );
	}

	function admin_queue() {
		add_thickbox();
		wp_enqueue_script( 'btr-admin', plugins_url( 'js/admin.js', __FILE__ ), array( 'thickbox' ) );
	}

	function register_settings() {
		register_setting( 'blogfa_settings', 'blogfa' );
	}

	function render() {
		$options = get_option( 'blogfa' );
		if( empty( $options['template'] ) )
			return;
		$blogfa_tags = array(
			'/<-BlogTitle->/',
			'/<-BlogDescription->/',
			'/<-BlogEmail->/',
			'/<-BlogUrl->/',
			'/<-BlogAuthor->/',
			'/<-BlogXmlLink->/',
			'/<-BlogAndPostTitle->/',
			'/<-BlogId->/',
			'/<BLOGFA>/',
			'/<\/BLOGFA>/',
			'/<-PostTitle->/',
			'/<-PostContent->/',
			'/<-PostAuthor->/',
			'/<-PostDate->/',
			'/<-PostTime->/',
			'/<-PostLink->/',
			'/<BlogExtendedPost>/',
			'/<\/BlogExtendedPost>/',
			'/<BlogNextAndPreviousBlock>/',
			'/<BlogPreviousPageBlock>/',
			'/<BlogNextPageBlock>/',
			'/<\/BlogNextAndPreviousBlock>/',
			'/<\/BlogPreviousPageBlock>/',
			'/<\/BlogNextPageBlock>/',
			'/<-BlogPreviousPageLink->/',
			'/<-BlogNextPageLink->/',
			'/<BlogProfile>/',
			'/<\/BlogProfile>/',
			'/<-BlogAbout->/',
			'/<-BlogCustomHtml->/',
		);
		$replaces = array(
			get_bloginfo('name'),
			get_bloginfo('description'),
			get_bloginfo('admin_email'),
			get_bloginfo('url'),
			$this->_get_admin_name(),
			get_bloginfo('rss_url'),
			'<?php echo ( is_singular() ) ? ( get_bloginfo("name") . " - " .get_the_title() ) : get_bloginfo("name"); ?>',
			'',
			'<?php while( have_posts() ) : the_post(); ?>',
			'<?php if( is_singular() ) comments_template() ; endwhile; ?>',
			'<?php the_title() ?>',
			'<?php if( is_singular() ) the_excerpt(); else the_content(); ?>',
			'<?php the_author() ?>',
			'<?php echo function_exists( "jdate" ) ? jdate( get_option("date_format"), strtotime( $post->post_date ) ) : get_the_date(); ?>', /* support for jalali calendar */
			'<?php the_time() ?>',
			'<?php the_permalink() ?>',
			'<?php if( ! is_singular() ) : ?>',
			'<?php endif; ?>',
			'<?php if ( $wp_query->max_num_pages > 1 ) : ?>',
			'<?php if ( $wp_query->max_num_pages > 1 ) : ?>',
			'<?php if ( $wp_query->max_num_pages > 1 ) : ?>',
			'<?php endif; ?>',
			'<?php endif; ?>',
			'<?php endif; ?>',
			'<?php echo get_previous_posts_page_link() ?>',
			'<?php echo get_next_posts_page_link() ?>',
			'',
			'',
			$options['about'],
			$options['html'],
		);
		$template = preg_replace( $blogfa_tags, $replaces, $options['template'] );

		/* add language_attributes to html element */
		$template = preg_replace( '/<html>/', '<html <?php language_attributes() ?>>', $template );

		/* comments */
		$template = preg_replace( '/<BlogComment>(.*?)<\/BlogComment>/', sprintf( '<?php comments_popup_link( "%s", "%s", "%s" ); ?>', __( 'Leave a comment', $this->textdomain ), __( '1 Comment', $this->textdomain ), __( '% Comments', $this->textdomain ) ), $template );

		/* BlogPreviousItemsBlock */
		$template = preg_replace( '/<BlogPreviousItemsBlock>/', '<?php if( have_posts() ) : ?>', $template );
		$template = preg_replace( '/<\/BlogPreviousItemsBlock>/', '<?php endif; ?>', $template );
		$template = preg_replace( '/<BlogPreviousItems\s([^>]*)?>/', '<?php query_posts("posts_per_page=10"); while( have_posts() ) : the_post(); ?>', $template );
		$template = preg_replace( '/<\/BlogPreviousItems>/', '<?php endwhile; rewind_posts(); ?>', $template );

		/* BlogAuthorsBlock */
		$template = preg_replace( '/<BlogAuthorsBlock>/', '<?php if( check_for_multi_author() ) : ?>', $template );
		$template = preg_replace( '/<\/BlogAuthorsBlock>/', '<?php endif; ?>', $template );
		$template = preg_replace( '/<BlogAuthors>(.*?)<\/BlogAuthors>/s', '<?php wp_list_authors("show_fullname=1&exclude_admin=0") ?>', $template );

		/**
		 * BlogCategories
		 * BlogCategoriesBlock is disabled since in WordPress, at least one category exists at a time.
		 */
		$template = preg_replace( '/<BlogCategories>(.*?)<\/BlogCategories>/s', '<?php wp_list_categories("title_li=") ?>', $template );
		$template = preg_replace( array( '/<BlogCategoriesBlock>/', '/<\/BlogCategoriesBlock>/' ), '', $template );

		/* BlogArchive */
		$template = preg_replace( '/<BlogArchive>(.*?)<\/BlogArchive>/s', '<?php wp_get_archives("") ?>', $template );

		/**
		 * BlogLinks & BlogLinkDump
		 * WP's link categories is disabled by default
		 * 
		 */
		if( isset( $options['daily'] ) && $options['daily'] !== 'none' ) {
			$template = preg_replace( '/<BlogLinkDumpBlock>/', '<?php $linksdump = get_categories("category='.$options['daily'] .'"); if( ! empty( $linksdump ) ) : ?>', $template );
			$template = preg_replace( '/<\/BlogLinkDumpBlock>/', '<?php endif; ?>', $template );
			$template = preg_replace( '/<BlogLinkDump>(.*?)<\/BlogLinkDump>/s', '<?php wp_list_bookmarks("title_li=&categorize=0&category= '. $options['daily'] .'") ?>', $template );
		} else {
			$template = preg_replace( '/<BlogLinkDumpBlock>(.*?)<\/BlogLinkDumpBlock>/s', '', $template );
			$template = preg_replace( '/<BlogLinkDump>(.*?)<\/BlogLinkDump>/s', '', $template );
		}
		if( isset( $options['links'] ) && $options['links'] !== 'none' ) {
			$template = preg_replace( '/<BlogLinksBlock>/', '<?php $links = get_categories("category='.$options['links'] .'"); if( ! empty( $links ) ) : ?>', $template );
			$template = preg_replace( '/<\/BlogLinksBlock>/', '<?php endif; ?>', $template );
			$template = preg_replace( '/<BlogLinks>(.*?)<\/BlogLinks>/s', '<?php wp_list_bookmarks("title_li=&categorize=0&category= '. $options['links'] .'") ?>', $template );
		} else {
			$template = preg_replace( '/<BlogLinksBlock>(.*?)<\/BlogLinksBlock>/s', '', $template );
			$template = preg_replace( '/<BlogLinks>(.*?)<\/BlogLinks>/s', '', $template );
		}

		/* blog photo */
		if( ! empty( $options['photo'] ) ) {
			$template = preg_replace( array( '/<BlogPhoto>/', '/<\/BlogPhoto>/' ), '', $template );
			$template = preg_replace( '/<-BlogPhotoLink->/', $options['photo'], $template );
		} else {
			$template = preg_replace( '/<BlogPhoto>(.*?)<\/BlogPhoto>/s', '', $template );
		}

		/* remove the comments script code */
		$template = preg_replace( '/\<!-- BeginCommentCode --\>(.*)\<!-- EndCommentCode --\>/s', '', $template );
		#$template = preg_replace( '/<script(.*?)function GetBC(.*?)<\/script>/s', '', $template );

		/* add wp_head & wp_footer hooks */
		$template = preg_replace( '/<\/head>/', '<?php wp_head() ?></head>', $template );
		$template = preg_replace( '/<\/body>/', '<?php wp_footer() ?></body>', $template );

		/* it's 20 times faster to define a dynamic function and call it, than to eval the code directly */
		#echo $template;
		#exit;
		eval( 'function btr_do_render(){ global $wp_query; ?>'. $template . '<?php }' );
		btr_do_render();

		/* stop WordPress from processing the theme */
		exit;
	}

	function _get_admin_name() {
		$user_info = get_userdata(1);
		return $user_info->display_name;
	}

	function _links_category_dropdown( $current = null ) {
		$c = get_categories( 'title_li=&taxonomy=link_category&hide_empty=0' );
		echo '<option value="none">'. __( 'None', $this->textdomain ) .'</option>';
		if( ! empty( $c ) ) : foreach( $c as $cat ) :
			echo '<option value="'. $cat->cat_ID .'"'. selected( $cat->cat_ID, $current ) .'>'. $cat->cat_name . '</option>';
		endforeach; endif;
	}
}
new Blogfa_Template_Renderer;

register_activation_hook( __FILE__, array( 'Blogfa_Template_Renderer', 'activation' ) );
register_deactivation_hook( __FILE__, array( 'Blogfa_Template_Renderer', 'deactivation' ) );